#!/usr/bin/python

import re, sys

repl_dict = {
#repl_dict#
}

def repl_func(matchobj):
    key = matchobj.group(0)[1:-1]
    if key in repl_dict:
        return repl_dict[key]
    else:
        return matchobj.group(0)


print re.sub('@\w+@', repl_func, sys.stdin.read())




#!/usr/bin/python

import subprocess as sp
import re, tempfile
import shutil, os, sys

##################################################
# Loging
import logging
def _init_log():
    name = 'app'
    x = logging.getLogger(name)
    x.setLevel(logging.WARNING)
    # x.setLevel(logging.DEBUG)
    h = logging.StreamHandler()
    # f = logging.Formatter("%(name)s (%(funcName)s:%(lineno)d) :: %(message)s")
    f = logging.Formatter("%(message)s")
    h.setFormatter(f)
    x.addHandler(h)
    return x

x = _init_log()

##################################################
# Argument parsing
import argparse
p = argparse.ArgumentParser(
    description='Pack project source to tar archive.',
    epilog = 'Example: \n%(prog)s /tmp/proj-1.2.tar.bz2'
)

p.add_argument('-v', dest = 'verbose', action = 'store_true',
    default = False,
    help = 'Print more messages [%(default)s]')
p.add_argument('-d', '--debug', dest = 'debug', action = 'store_true',
    default = False,
    help = 'Print debug messages [%(default)s]')
p.add_argument('tar', nargs=1,
    help = 'tar file to create')

args = p.parse_args()
if args.verbose:
    x.setLevel(logging.INFO)
if args.debug:
    x.setLevel(logging.DEBUG)

##################################################
# main
def my_check_output(*popenargs, **kwargs):  
    if 'stdout' in kwargs:
        raise ValueError('stdout argument not allowed, it will be overridden.')
    cmd = kwargs.get("args")
    if cmd is None:
        cmd = popenargs[0]
    x.debug("exec: %s", cmd)
    process = sp.Popen(stdout=sp.PIPE, *popenargs, **kwargs)
    output, unused_err = process.communicate()
    retcode = process.poll()
    if retcode:
        raise sp.CalledProcessError(retcode, cmd, output=output)
    return output

def svn_is_used():
    f = open('/dev/null', 'w')
    p = sp.Popen('svn info'.split(), stdout = f, stderr = f)
    p.wait()
    return p.returncode == 0


def svn_get_file_list():
    text = my_check_output('svn info -R'.split())
    files = []
    tre = '(?m)^Path: (?P<path>.*)$(.|\s)*?^Node Kind: (?P<type>.*)$'
    for m in re.finditer(tre, text):
        if m.group('type') == 'file':
            files.append(m.group('path'))
    return files

def git_is_used():
    f = open('/dev/null', 'w')
    p = sp.Popen('git status'.split(), stdout = f, stderr = f)
    p.wait()
    return p.returncode == 0


def git_get_file_list():
    text = my_check_output('git ls-tree --name-only -r HEAD'.split())
    files = text.split('\n')
    return files

def none_is_used():
    f = open('/dev/null', 'w')
    p = sp.Popen('make -n distclean'.split(), stdout = f, stderr = f)
    p.wait()
    return p.returncode == 0
    
def none_get_file_list():
    f = open('/dev/null', 'w')
    p = sp.Popen('make distclean'.split(), stdout = f, stderr = f)
    p.wait()
    if p.returncode:
        return []
    files = []
    for (dirpath, dirnames, filenames) in os.walk('.'):
        if dirpath.startswith('./'):
            dirpath = dirpath[2:]
        elif dirpath == '.':
            dirpath = ''
        filenames = [ os.path.join(dirpath, f) for f in filenames ]
        files.extend(filenames)
    return files

vcs = [
    {
        'name' : 'svn',
        'is_used' : svn_is_used,
        'get_file_list' : svn_get_file_list
    },
    {
        'name' : 'git',
        'is_used' : git_is_used,
        'get_file_list' : git_get_file_list
    },
    
    # must be last entry
    {
        'name' : 'none',
        'is_used' : none_is_used,
        'get_file_list' : none_get_file_list
    }
]

# Get lists of files. If VCS is used, then lists only controlled files
# otherwise lists all files
def get_file_list():
    files = []
    for v in vcs:
        ret = v['is_used']()
        x.debug('trying %s, ret %d', v['name'], ret)
        if ret:
            x.info('VCS: %s', v['name'])
            files = v['get_file_list']()
            # remove duplicates, if any
            files = list(set(files))
            files.remove('')
            files.sort()
            x.debug('File list: %s', files)
            if not files:
                x.error('No files. Aborting')
                exit(2)
            return files
    x.error('No files. Aborting')
    exit(2)

tar = args.tar[0]
path, name = os.path.split(tar)
nv = re.sub('.tar.*', '', name)
x.debug('tar %s, nv %s', tar, nv)
# XXX: get compresion type from extension

files = [ nv + '/' + f for f in get_file_list() ]
files = '\n'.join(files)
x.debug('%s', files)
tdir = tempfile.mkdtemp()
odir = os.path.abspath('.')
os.chdir(tdir)
os.symlink(odir, nv)
open('filelist', 'w').write(files)
cmd = 'tar hjcf %s --files-from filelist' % (tar,)
x.debug('exec: %s', cmd)
p = sp.Popen(cmd.split())
p.wait()
if p.returncode:
    x.error('Aborting')
else:
    x.info('Tar: %s', tar)

shutil.rmtree(tdir)




# Initialization. Create your opts here. Runs before command line processing.
def init():
    # standart autoconf options
    opt_group_new('autoconf', 'Standart autoconf options')
    opt_new("prefix", group = 'autoconf',
        help = "install architecture-independent files", metavar='DIR',
        default = lambda : '/usr')
    opt_new("eprefix", group = 'autoconf',
        help = "install architecture-dependent files", metavar='DIR',
        default = lambda : opt('prefix'))
    opt_new("bindir", group = 'autoconf',
        help = "user executables", metavar='DIR',
        default = lambda : opt('eprefix') + '/bin')
    opt_new("sbindir", group = 'autoconf',
        help = "system executables", metavar='DIR',
        default = lambda : opt('eprefix') + '/sbin')
    opt_new("libexecdir", group = 'autoconf',
        help = "program executables", metavar='DIR',
        default = lambda : opt('eprefix') + '/libexec/' + opt('project_name'))
    opt_new("libdir", group = 'autoconf',
        help = "object code libraries", metavar='DIR',
        default = lambda : opt('eprefix') + '/lib/' + opt('project_name'))
    opt_new("sysconfdir", group = 'autoconf',
        help = "read-only single-machine data", metavar='DIR',
        default = lambda : opt('prefix') + '/etc')
    opt_new("datadir", group = 'autoconf',
        help = "read-only architecture-independent data", metavar='DIR',
        default = lambda : opt('prefix') + '/share/' + opt('project_name'))
    opt_new("localedir", group = 'autoconf',
        help = "languagetranslation files", metavar='DIR',
        default = lambda : opt('datadir') + '/locale')
    opt_new("includedir", group = 'autoconf',
        help = "C header files", metavar='DIR',
        default = lambda : opt('prefix') + '/include')
    opt_new("mandir", group = 'autoconf',
        help = "man documentation", metavar='DIR',
        default = lambda : opt('prefix') + '/man')
    opt_new("infodir", group = 'autoconf',
        help = "info documentation", metavar='DIR',
        default = lambda : opt('prefix') + '/info')
    opt_new("localstatedir", group = 'autoconf',
        help = "modifiable single-machine data in DIR", metavar='DIR',
        default = lambda : opt('prefix') + '/var')
    
    opt_group_new('app', 'Application options', default = True)
    opt_new("project_name", group = 'app',
        help = "Project name", metavar='NAME',
        default = 'fbpanel')
    opt_new("project_version", group = 'app',
        help = "Project version", metavar='VER',
        default = lambda : detect_project_version())
    opt_new("sound", group = 'app',
        help = "Enable sound plugin (default: autodetect)",
        action = ToggleAction, default = None)

# Logic and stuff that does not require human interaction. Runs after
# command line processing.
# Here you can do anything you want. You can create opts, delete opts, modify
# their values. Just bear in mind: whatever you do here will end up in
# config.h.
def resolve():
    # If user did not set sound on command line, it is autodetected.
    if opt('sound') is None and pkg_exists('alsa', '--atleast-version=1.0.10'):
        opt_set('sound', True)

    # alsa is required, only if "sound" is enabled.
    if opt('sound'):
        # if alsa is not installed, will raise exception
        opt_new_from_pkg('alsa', 'alsa', pversion = '--atleast-version=1.0.10')

    opt_new_from_pkg('gtk2', 'gtk+-2.0', pversion = '--atleast-version=2.17')
    opt_new_from_pkg('gmodule2', 'gmodule-2.0')
    opt_new_from_pkg('x11', 'x11')
    opt_new('cflags_extra', default='-I$(TOPDIR)/panel')

def detect_project_name():
    # Hardcode here the name of your project
    # ret = "projectname"

    # Alternatively, take top dir name as a project name
    ret = os.getcwd().split('/')[-1]

    return ret

def detect_project_version():
    ret = open('version', 'r').read().strip()

    return ret

# Give a summary on created configuration
def report():
    str = "Configuration:\n"
    str += "  Sound plugin:  "
    if opt('sound'):
        str += "yes\n"
    else:
        str += "no\n"
    print str,

